#
# spec file for package lfortran
#
# Copyright (c) 2024 SUSE LLC
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#

# for static libraries
%global _lto_cflags %{?_lto_cflags} -ffat-lto-objects

Version:        0.31.0
%global         sover 0
Name:           lfortran
Release:        1.3
Summary:        A modern interactive Fortran compiler built on top of LLVM

# Main code is BSD-3-Clause
# src/libasr/codegen/KaleidoscopeJIT.h is available under the Apache 2.0
# License with LLVM exception
License:        BSD-3-Clause AND Apache-2.0 WITH LLVM-exception
URL:            https://lfortran.org/
Source0:        https://lfortran.github.io/tarballs/release/lfortran-%{version}.tar.gz

# https://github.com/lfortran/lfortran/issues/2981
ExclusiveArch: x86_64

BuildRequires: binutils-devel
BuildRequires: bison
BuildRequires: cmake
BuildRequires: fmt-devel
BuildRequires: gcc-c++ >= 8
BuildRequires: nlohmann_json-devel
BuildRequires: kokkos-devel
# kokkos is not link, but only use for backend=cpp
Requires: kokkos-devel
BuildRequires: libffi-devel
BuildRequires: libunwind-devel
BuildRequires: libuuid-devel
BuildRequires: llvm16-devel
BuildRequires: python3-devel
BuildRequires: rapidjson-devel
BuildRequires: re2c
BuildRequires: zlib-devel
BuildRequires: zlib-devel-static
BuildRequires: libzstd-devel
BuildRequires: libzstd-devel-static

%global lfortran_desc \
LFortran is a modern open-source (BSD licensed) interactive Fortran \
compiler built on top of LLVM. It can execute user's code interactively \
to allow exploratory work (much like Python, MATLAB or Julia) as well as \
compile to binaries with the goal to run user's code on modern \
architectures such as multi-core CPUs and GPUs.

%description
%{lfortran_desc}

%package devel
Summary:  Development headers and libraries for %{name}
Requires: %{name}%{?_isa} = %{version}-%{release}
Requires:  liblfortran%{sover}%{?_isa} = %{version}-%{release}

%description devel
%{lfortran_desc}

This package contains development headers and libraries for %{name}.

%package -n liblfortran%{sover}
Summary:   %{name} runtime library

%description -n liblfortran%{sover}
%{lfortran_desc}

This package contains shared runtime libraries of %{name}.


%package devel-static
Summary:   %{name} static runtime library
Requires:  %{name}-devel = %{version}

%description devel-static
%{lfortran_desc}

This package contains static runtime library for %{name}.


%prep
%autosetup -p1

%build
%cmake -DCMAKE_PREFIX_PATH=%{_libdir}/llvm16/ \
       -DWITH_LLVM=ON \
       -DWITH_RUNTIME_LIBRARY=ON \
       -DWITH_FMT=ON \
       -DWITH_JSON=ON \
       -DWITH_KOKKOS=ON \
       -DWITH_STACKTRACE=OFF \
       -DWITH_UNWIND=ON \
       -DWITH_WHEREAMI=OFF \
       -DWITH_XEUS=OFF \
       -DWITH_ZLIB=ON \
       ..
%cmake_build

%install
%cmake_install

%check
%ctest

%ldconfig_scriptlets -n liblfortran%sover

%files
%doc README.md
%{_bindir}/lfortran
%{_mandir}/man1/lfortran.1.*

%files -n liblfortran%{sover}
%license LICENSE
%{_libdir}/liblfortran_runtime.so.%{sover}*

%files devel
%dir %{_includedir}/lfortran
%dir %{_includedir}/lfortran/impure
%{_includedir}/lfortran/impure/lfortran_intrinsics.h
%{_libdir}/liblfortran_runtime.so
%{_libdir}/lfortran_*.mod

%files devel-static
%{_libdir}/liblfortran_runtime_static.a

%changelog
* Mon Jan 22 2024 Christoph Junghans <junghans@votca.org>
- Update to v0.31.0
  - CI: add lfortran.1 to tarball ([gh#lfortran/lfortran##3041])
  - Update generic travel example ([gh#lfortran/lfortran##2995])
  - Implement intrinsic repeat ([gh#lfortran/lfortran##3038])
  - Warning when the LHS character len and the RHS character len are not equal
    ([gh#lfortran/lfortran##3005])
  - CI: test example of stdlib special functions gamma
    ([gh#lfortran/lfortran##3036])
  - Add gamma to IntrinsicScalarFunction ([gh#lfortran/lfortran##3032])
  - ASR: Print => as Associate and = as Assignment
    ([gh#lfortran/lfortran##3040])
  - cmake: add libm to lfortran_runtime ([gh#lfortran/lfortran##3043])
  - Support calling scalar intrinsics with allocatable inputs
    ([gh#lfortran/lfortran##3049])
  - ci: update stdlib branch to remove workaround for gamma
    ([gh#lfortran/lfortran##3045])
  - Added tsunami integration test ([gh#lfortran/lfortran##3055])
  - Fixes for compiling testdrive.F90 ([gh#lfortran/lfortran##3062])
  - FIX: Type mismatch in array initializer r32 and i32
    ([gh#lfortran/lfortran##3059])
  - FIX: operands to a binary operator are not of the same type!
    ([gh#lfortran/lfortran##3061])
  - Handle template with elemental functions ([gh#lfortran/lfortran##2895])
  - Fixes for bugs in testdrive.F90 ([gh#lfortran/lfortran##3065])
  - Stdlib: Fix compile time evaluation for char() with IntegerBOZ
    ([gh#lfortran/lfortran##3064])
  - Fix allocatable to non-allocatable arguments in testdrive.F90
    ([gh#lfortran/lfortran##3068])
  - Rename IntrinsicFunctionSqrt to RealSqrt ([gh#lfortran/lfortran##3050])
  - Stdlib: Support evaluating len() at compile-time
    ([gh#lfortran/lfortran##3077])
  - Simplify ASR.asdl ([gh#lfortran/lfortran##3063])
  - Emit warning for complex*4 x ([gh#lfortran/lfortran##3066])
  - Support char type for write(..) ([gh#lfortran/lfortran##2888])
  - FIX: inconsistent kind handling for array item indices
    ([gh#lfortran/lfortran##3093])
  - Nested template definitions and instantiations
    ([gh#lfortran/lfortran##2996])
  - Stdlib: Fix initialization for complex parameter
    ([gh#lfortran/lfortran##3097])
  - Emit warning for logical* 4 ([gh#lfortran/lfortran##3099])
  - Stdlib: LFortran compiles ascii ([gh#lfortran/lfortran##3095])
  - Fix complex parameter initialization ([gh#lfortran/lfortran##3100])
  - Fix shape intrinsic for scalar arg ([gh#lfortran/lfortran##3094])
  - Fix use of keyword args for intrinsic functions
    ([gh#lfortran/lfortran##3075])
  - doc: Update Windows build instructions ([gh#lfortran/lfortran##3101])
  - Stdlib: LFortran compiles version and optval modules
    ([gh#lfortran/lfortran##3118])
  - Stdlib: Fix stop and error stop, Support stdlib_error module
    ([gh#lfortran/lfortran##3121])
  - Refactor code ([gh#lfortran/lfortran##3126])
  - ENH: support intrinsic dfloat ([gh#lfortran/lfortran##3131])
  - Handle kind for trailz ([gh#lfortran/lfortran##3132])
  - Implement shiftr ([gh#lfortran/lfortran##3134])
  - Compiling test/array to ASR ([gh#lfortran/lfortran##3135])
  - Corrected keywords for intrinsic functions ([gh#lfortran/lfortran##3123])
  - implement leadz ([gh#lfortran/lfortran##3141])
  - Improve intrinsic tests ([gh#lfortran/lfortran##3145])
  - Implement rank ([gh#lfortran/lfortran##3146])
  - Python script for Intrinsic Functions verify_args()
    ([gh#lfortran/lfortran##3150])
  - cmake: allow llvm without zstd ([gh#lfortran/lfortran##3157])
- Dropped 3043.patch, merged upsteam
* Tue Jan  9 2024 Christoph Junghans <junghans@votca.org>
- Add kokkos dep
* Thu Jan  4 2024 Christoph Junghans <junghans@votca.org>
- Added upstream patch, 3043.patch, to fix linking on openSUSE:Factory
* Wed Jan  3 2024 Christoph Junghans <junghans@votca.org>
- initial import
